"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.deployer = exports.EDeployMode = void 0;
const fs_extra_1 = __importDefault(require("fs-extra"));
const archiver_1 = __importDefault(require("archiver"));
const miniprogram_ci_1 = require("miniprogram-ci");
const qrcode_1 = __importDefault(require("qrcode"));
const path_1 = __importDefault(require("path"));
const cloud_api_1 = require("@cloudbase/cloud-api");
const cloud_api_service_1 = require("./utils/tce-cloud-api/cloud_api_service");
const manager_node_1 = __importDefault(require("@cloudbase/manager-node"));
const parallel_1 = require("@cloudbase/manager-node/lib/utils/parallel");
const private_1 = require("./config/private");
const static_builder_1 = require("@cloudbase/static-builder");
const emoji_1 = require("./utils/emoji");
const lowcode_builder_1 = require("@cloudbase/lowcode-builder");
const storage_1 = require("./utils/storage");
const url_1 = __importDefault(require("url"));
const NOT_NPM_ERROR = '__NO_NODE_MODULES__ NPM packages not found';
const QRCODE_PATH = './qrcode.jpg';
class Deployer {
    constructor(options) {
        // this.app = options.cloudbaseManager;
        // this.logger = getLogger();
        this.logger = options.logger || console;
        this.api = options.api || { emoji: emoji_1.emoji };
    }
}
var EDeployMode;
(function (EDeployMode) {
    EDeployMode["PRODUCTION"] = "PRODUCTION";
    EDeployMode["PREVIEW"] = "PREVIEW";
})(EDeployMode = exports.EDeployMode || (exports.EDeployMode = {}));
class LowcodeDeployer extends Deployer {
    constructor(options) {
        super(options);
        this.credential = options.credential;
        this.resolvedInputs = this.normalizInputs(options);
        const { type, mpAppId, projectPath, ignores, miniprogramOptions } = this.resolvedInputs;
        if (type === 'MP') {
            this.mpCIProject =
                (miniprogramOptions === null || miniprogramOptions === void 0 ? void 0 : miniprogramOptions.ciProject) ||
                    this._generateMpCIProject({
                        mpAppId,
                        projectPath,
                        privateKeyPath: miniprogramOptions === null || miniprogramOptions === void 0 ? void 0 : miniprogramOptions.privateKeyPath,
                        ignores: ['node_modules/**/*', ...ignores],
                        ...miniprogramOptions === null || miniprogramOptions === void 0 ? void 0 : miniprogramOptions.projectConfig,
                    });
        }
        else {
            this.lowcodeService = this._generateLowcodeService(this.credential);
            this.tcbService = this._generateTcbService(this.credential);
            this.cloudbaseManager = new manager_node_1.default({
                ...this.credential,
                envId: this.resolvedInputs.envId,
            });
        }
    }
    normalizInputs(options) {
        var _a, _b, _c;
        const resolvedInputs = {
            ...options,
            outputPath: options.outputPath || options.projectPath,
            ignores: options.ignores || [],
        };
        if (resolvedInputs.type === 'MP') {
            const { miniprogramOptions = {}, projectPath } = resolvedInputs;
            let { privateKey, privateKeyPath = '' } = miniprogramOptions;
            if (privateKey) {
                const keyPath = path_1.default.resolve(projectPath, './private.key');
                const key = Buffer.from(privateKey, 'base64');
                fs_extra_1.default.writeFileSync(keyPath, key.toString());
                privateKeyPath = keyPath;
            }
            if (!privateKeyPath || !fs_extra_1.default.existsSync(path_1.default.resolve(projectPath, privateKeyPath))) {
                throw new Error('need privateKey or privateKey in miniprogramOptions config');
            }
            miniprogramOptions.privateKeyPath = privateKeyPath;
            resolvedInputs.miniprogramOptions = miniprogramOptions;
        }
        else if (resolvedInputs.type === 'WEB') {
            const requiredValues = ['appId', 'envId', 'credential'];
            for (let key of requiredValues) {
                if (!resolvedInputs[key]) {
                    throw new Error(`web 部署模式缺乏必须参数 ${key}`);
                }
            }
            let { webOptions = {} } = resolvedInputs;
            if (this.isPrivate && !webOptions.credential) {
                throw new Error('web 私有化部署缺乏必须参数 webOptions.credential');
            }
            webOptions.cloudPath = this._getWebRootPath(resolvedInputs);
            if (!webOptions.buildTypeList) {
                webOptions.buildTypeList = [];
            }
            resolvedInputs.webOptions = webOptions;
        }
        if (resolvedInputs.artifactOptions) {
            if (!resolvedInputs.artifactOptions.baseKey) {
                throw new Error('缺少制品持久化目标地址配置');
            }
            if (!resolvedInputs.artifactOptions.bucket) {
                throw new Error('缺少制品持久存储桶配置');
            }
            resolvedInputs.artifactOptions = {
                ...resolvedInputs.artifactOptions,
                credential: resolvedInputs.artifactOptions.credential ||
                    (this.isPrivate ? (_a = resolvedInputs.webOptions) === null || _a === void 0 ? void 0 : _a.credential : this.credential),
                endpoint: resolvedInputs.artifactOptions.endpoint || ((_b = resolvedInputs.webOptions) === null || _b === void 0 ? void 0 : _b.endpoint),
                region: resolvedInputs.artifactOptions.region || ((_c = resolvedInputs.webOptions) === null || _c === void 0 ? void 0 : _c.region),
            };
        }
        return resolvedInputs;
    }
    get isPrivate() {
        return process.env.IS_PRIVATE === 'true';
    }
    _getWebRootPath(inputs) {
        let { appId, mode } = inputs;
        return mode === EDeployMode.PREVIEW ? `/${appId}/preview/` : `/${appId}/production/`;
    }
    async deploy(options = {}) {
        var _a, _b, _c, _d, _e;
        const { type, mode } = this.resolvedInputs;
        try {
            if (type === 'MP') {
                try {
                    await this.genearteMiniprogramNPM();
                    const list = await this._cleanNonRefFile();
                    (_b = (_a = this.logger) === null || _a === void 0 ? void 0 : _a.info) === null || _b === void 0 ? void 0 : _b.call(_a, list.join('/n'));
                }
                catch (e) {
                    (_d = (_c = this.logger) === null || _c === void 0 ? void 0 : _c.debug) === null || _d === void 0 ? void 0 : _d.call(_c, e.message);
                }
                return mode === EDeployMode.PRODUCTION ? this._ciUpload(options) : this._ciPreview(options);
            }
            else if (type === 'WEB') {
                const { webOptions } = this.resolvedInputs;
                try {
                    await this.lowcodeService.request('UpdateLoginConfig', { EnvId: this.resolvedInputs.envId });
                    (_e = this.logger) === null || _e === void 0 ? void 0 : _e.debug('login config set success');
                }
                catch (e) {
                    this.logger.error(`设置登录配置失败 ${e.message}}`);
                }
                const { website, domains = [] } = await this._preProcessWebDeploy();
                if (!(0, lowcode_builder_1.buildAsXPageByBuildType)(webOptions.buildTypeList)) {
                    const ignores = Array.from(new Set(['.tmp', '.git', '.github', 'node_modules', 'cloudbaserc.js', ...this.resolvedInputs.ignores]));
                    const includes = ['**', ...ignores.map((ignore) => `!${ignore}`)];
                    const builder = new static_builder_1.StaticBuilder({
                        projectPath: path_1.default.resolve(this.resolvedInputs.projectPath),
                    });
                    const { static: staticList = [], staticConfig = [] } = await builder.build(includes, {
                        path: this.resolvedInputs.webOptions.cloudPath,
                        domain: this.isPrivate ? undefined : website === null || website === void 0 ? void 0 : website.CdnDomain,
                    });
                    const deployContent = [...staticList];
                    if (this.isPrivate) {
                        const { endpoint, region, bucket, credential } = webOptions;
                        const instance = new storage_1.Storage({ endpoint, credential, region, cos: false });
                        const fileList = deployContent.reduce((list, meta) => {
                            let { cloudPath } = meta;
                            if (cloudPath.startsWith('/')) {
                                cloudPath = cloudPath.substring(1);
                            }
                            list.push(...(0, storage_1.getUploadDirFileList)(meta.src, cloudPath));
                            return list;
                        }, []);
                        const asyncTaskController = new parallel_1.AsyncTaskParallelController(20, 50);
                        asyncTaskController.loadTasks(fileList.map((meta) => {
                            return async () => {
                                try {
                                    const param = {
                                        Bucket: bucket,
                                        ACL: 'public-read',
                                        Body: fs_extra_1.default.createReadStream(meta.src),
                                        Key: meta.cloudPath,
                                    };
                                    try {
                                        await instance.putObject(param);
                                    }
                                    catch (e) {
                                        // 间隔重试一次
                                        await new Promise((resolve, reject) => {
                                            setTimeout(() => {
                                                resolve(null);
                                            }, 500);
                                        });
                                        await instance.putObject(param);
                                    }
                                    return { code: 0 };
                                }
                                catch (e) {
                                    if (!e.code) {
                                        e.code = -1;
                                    }
                                    throw e;
                                }
                            };
                        }));
                        const list = await asyncTaskController.run();
                        if (list.some((item) => item.code)) {
                            console.log(list.filter((item) => item.code));
                            throw new Error('部分文件部署失败');
                        }
                    }
                    else {
                        /**
                         * 私有化场景该文件交给初始化生成，不再兜底
                         * 只有公有场景下防止环境未初始化情形，进行一次兜底
                         */
                        // 写入私有化JS文件占位
                        const privateJsPath = path_1.default.join(this.resolvedInputs.projectPath, '.tmp', private_1.PRIVATE_JS_NAME);
                        try {
                            await fs_extra_1.default.ensureFile(privateJsPath);
                            await fs_extra_1.default.writeFile(privateJsPath, private_1.PRIVATE_JS_CODE, {
                                encoding: 'utf8',
                                flag: 'w',
                            });
                        }
                        catch (e) {
                            this.logger.error('创建私有化JS文件失败:', e);
                            throw e;
                        }
                        deployContent.push(...staticConfig);
                        deployContent.push({ src: privateJsPath, cloudPath: `/weda-config/${private_1.PRIVATE_JS_NAME}` });
                        await Promise.all(deployContent.map((options) => this.cloudbaseManager.hosting.uploadFiles({
                            localPath: options.src,
                            cloudPath: options.cloudPath,
                            ignore: ignores,
                        })));
                        try {
                            await fs_extra_1.default.remove(privateJsPath);
                        }
                        catch (e) { }
                    }
                    try {
                        await builder.clean();
                    }
                    catch (e) { }
                }
                else {
                    this.logger.info('xpage 跳过部署流程');
                }
                await this._postProcessWebDeploy({ domains });
            }
        }
        catch (e) {
            throw e;
        }
        finally {
            await this._persistArtifactOptions();
        }
    }
    async _persistArtifactOptions() {
        if (this.resolvedInputs.artifactOptions) {
            const { endpoint, bucket, region, baseKey, credential } = this.resolvedInputs.artifactOptions;
            try {
                const tmpPath = path_1.default.resolve('.tmp');
                fs_extra_1.default.ensureDirSync(path_1.default.resolve('.tmp'));
                fs_extra_1.default.ensureDirSync(this.resolvedInputs.projectPath);
                const zipPath = path_1.default.resolve('.tmp', `${this.resolvedInputs.appId}.zip`);
                await zipDir(path_1.default.resolve(this.resolvedInputs.projectPath), zipPath);
                const storage = new storage_1.Storage({
                    endpoint,
                    region,
                    credential: credential,
                    cos: !this.isPrivate,
                });
                await storage.putObject({
                    ACL: 'public-read',
                    Bucket: bucket || '',
                    Key: path_1.default.posix.join(baseKey, 'dist.zip'),
                    Body: fs_extra_1.default.createReadStream(zipPath),
                });
                fs_extra_1.default.removeSync(zipPath);
                if (fs_extra_1.default.existsSync(path_1.default.resolve(this.resolvedInputs.outputPath, QRCODE_PATH))) {
                    await storage.putObject({
                        ACL: 'public-read',
                        Bucket: bucket || '',
                        Key: path_1.default.posix.join(baseKey, 'qrcode.jpg'),
                        Body: fs_extra_1.default.createReadStream(path_1.default.resolve(this.resolvedInputs.outputPath, QRCODE_PATH)),
                    });
                }
                this.logger.info(`${this.api.emoji('🚀')} 持久化制品成功。`);
            }
            catch (e) {
                this.logger.error(`${this.api.emoji('🚀')} 持久化上传制品失败：`, e);
            }
        }
    }
    async _preProcessWebDeploy() {
        const { envId, webOptions = {} } = this.resolvedInputs;
        if (this.isPrivate) {
            const { domain } = webOptions;
            return { domains: [domain], website: { CdnDomain: domain } };
        }
        let domains, currentWebsite;
        try {
            let [website, hostingDatas] = await this._getHostingInfoTask(envId, false);
            if (!website) {
                throw new Error('静态托管初始化超时');
            }
            domains = hostingDatas.map((item) => item.CdnDomain);
            currentWebsite = website;
        }
        catch (e) {
            this.logger.error('获取静态托管失败: ', e);
            throw e;
        }
        return { domains, website: currentWebsite };
    }
    async _postProcessWebDeploy({ domains = [] }) {
        const { mode, appId, envId, webOptions, outputPath } = this.resolvedInputs;
        if ((0, lowcode_builder_1.buildAsAdminPortalByBuildType)(webOptions.buildTypeList)) {
            await this._postProcessAdminPortal();
        }
        else {
            await this._postProcessWebsiteConfig(domains);
        }
        try {
            let publishAppCustomNavRes = await this.lowcodeService.request('PublishAppCustomNav', {
                WeAppId: appId,
                EnvId: envId,
                PublishType: mode === EDeployMode.PRODUCTION ? 'product' : 'preview',
            });
            this.logger.info(`${this.api.emoji('🚀')} custom publish success: ${JSON.stringify(publishAppCustomNavRes)}`);
        }
        catch (e) {
            this.logger.error(`${this.api.emoji('🚀')} custom publish fail: ${JSON.stringify(e)}`);
            if ((e === null || e === void 0 ? void 0 : e.code) !== 'ResourceNotFound' && !/app does not exist/.test(e === null || e === void 0 ? void 0 : e.message)) {
                throw e;
            }
        }
        const defaultCdnDomain = domains[0];
        try {
            const isPreview = mode === EDeployMode.PREVIEW;
            const link = (0, lowcode_builder_1.buildAsAdminPortalByBuildType)(webOptions.buildTypeList)
                ? `https://${defaultCdnDomain}/adminportal/#/app/${isPreview ? `${appId}-preview` : appId}?envType=${isPreview ? 'preview' : 'prod'}`
                : webOptions.gatewayConfig
                    ? `https://${webOptions.gatewayConfig.domain + webOptions.gatewayConfig.path}`
                    : `https://${defaultCdnDomain + webOptions.cloudPath}`;
            const qrcodeOutputPath = path_1.default.resolve(outputPath, QRCODE_PATH);
            await qrcode_1.default.toFile(qrcodeOutputPath, link, {
                errorCorrectionLevel: 'M',
                type: 'image/jpeg',
                scale: 12,
                margin: 2,
            });
            this.logger.info(`${this.api.emoji('🚀')} 网站部署成功：${link}`);
            this.logger.info(`${this.api.emoji('🚀')} 网站部署成功, 访问二维码地址：${url_1.default.format({
                protocol: 'file:',
                host: qrcodeOutputPath,
            })}`);
        }
        catch (e) {
            this.logger.error('网站部署失败: ', e);
            throw e;
        }
    }
    async _postProcessAdminPortal() {
        if (this.isPrivate) {
            return;
        }
        try {
            if (this.resolvedInputs.mode === EDeployMode.PRODUCTION) {
                await this.lowcodeService.request('PublishAppCustomUrl', {
                    EnvId: this.resolvedInputs.envId,
                    WeAppId: this.resolvedInputs.appId,
                });
            }
            this.logger.debug('PublishAppCustomUrl成功');
        }
        catch (e) {
            this.logger.error('PublishAppCustomUrl失败: ', e);
        }
    }
    async _postProcessWebsiteConfig(domains) {
        if (this.isPrivate) {
            return;
        }
        const { mode, appId, envId, webOptions } = this.resolvedInputs;
        const hostingService = this.cloudbaseManager.hosting;
        try {
            let { Domains: domainList } = await hostingService.tcbCheckResource({
                domains,
            });
            // hostingService.getInfo();
            await Promise.all(domainList
                .filter((item) => item.DomainConfig.FollowRedirect !== 'on')
                .map((item) => hostingService.tcbModifyAttribute({
                domain: item.Domain,
                domainId: item.DomainId,
                domainConfig: { FollowRedirect: 'on' },
            })));
            await Promise.all([
                mode === EDeployMode.PRODUCTION
                    ? this.lowcodeService
                        .request('PublishAppCustomUrl', {
                        EnvId: envId,
                        WeAppId: appId,
                        IndexPath: webOptions.cloudPath,
                    })
                        .catch((e) => {
                        this.logger.error('自定义域名发布失败: ', e);
                        if ((e === null || e === void 0 ? void 0 : e.code) !== 'ResourceNotFound') {
                            throw e;
                        }
                    })
                    : Promise.resolve(),
            ]);
        }
        catch (e) {
            this.logger.error('网站路由注册失败: ', e);
            throw e;
        }
    }
    async _getHostingInfoTask(envId, loose = false) {
        let timeout = null;
        let [website, hostingDatas] = await Promise.race([
            new Promise((resolve) => {
                timeout = setTimeout(() => {
                    resolve([]);
                }, 300 * 1000);
            }),
            this._getHostingInfo(envId, loose),
        ]);
        if (timeout) {
            clearTimeout(timeout);
        }
        return [website, hostingDatas];
    }
    async _getHostingInfo(envId, loose = false) {
        let [website, hostingDatas] = await this.tcbService
            .request('DescribeStaticStore', { EnvId: envId })
            .then(({ Data: hostingDatas }) => {
            let website = hostingDatas[0];
            return [website, hostingDatas];
        });
        // 此处采取松校验
        if (!website || (loose ? false : (website === null || website === void 0 ? void 0 : website.Status) !== 'online')) {
            await new Promise((resolve) => {
                setTimeout(() => {
                    resolve(true);
                }, 8 * 1000);
            });
            return this._getHostingInfo(envId, loose);
        }
        else {
            return [website, hostingDatas];
        }
    }
    _generateLowcodeService(credential) {
        const params = {
            service: 'lowcode',
            version: '2021-01-08',
            timeout: 10000,
            credential,
        };
        return this.isPrivate
            ? new cloud_api_service_1.TCECloudApiService({
                ...params,
                baseUrl: process.env.PRIVATE_CLOUD_API_DOMAIN || 'yunapi3.weda-private.tcs230.fsphere.cn',
                endpoint: process.env.PRIVATE_ENDPOINT || '',
            })
            : new cloud_api_1.CloudApiService(params);
    }
    _generateTcbService(credential) {
        const params = {
            service: 'tcb',
            version: '2018-06-08',
            credential,
        };
        return this.isPrivate
            ? new cloud_api_service_1.TCECloudApiService({
                ...params,
                baseUrl: process.env.PRIVATE_CLOUD_API_DOMAIN || 'yunapi3.weda-private.tcs230.fsphere.cn',
                endpoint: process.env.PRIVATE_ENDPOINT || '',
            })
            : new cloud_api_1.CloudApiService(params);
    }
    _generateMpCIProject({ mpAppId: appid, projectPath, privateKeyPath, ignores }) {
        return new miniprogram_ci_1.Project({
            appid,
            type: 'miniProgram',
            projectPath,
            privateKeyPath,
            ignores,
        });
    }
    /**
     * 小程序-编译NPM
     */
    async genearteMiniprogramNPM() {
        // 需要暂时关掉 stdout, 避免 miniprogram-ci 的内容打印到控制台
        // this.api.runtime.isLocal() && this.api.console.pause();
        !process.env.CLOUDBASE_CIID && pauseConsoleOutput();
        const result = await (0, miniprogram_ci_1.packNpm)(this.mpCIProject, {
            reporter: (infos) => {
                console.log(infos);
            },
        }).catch((err) => {
            return err;
        });
        // this.api.runtime.isLocal() && this.api.console.resume();
        !process.env.CLOUDBASE_CIID && resumeConsoleOutput();
        if (result instanceof Error && !result.message.startsWith(NOT_NPM_ERROR)) {
            throw new Error(`小程序 NPM 构建失败 ${result}`);
        }
    }
    async _cleanNonRefFile() {
        const analyseRes = await (0, miniprogram_ci_1.analyseCode)(this.mpCIProject);
        if (analyseRes) {
            const codeExtensions = new Set(['.wxml', '.wxss', '.wxs', '.js', '.json', '.json', '.ts']);
            const { files } = analyseRes;
            const filesToDelete = files
                .filter((f) => {
                var _a, _b;
                if (codeExtensions.has(f.ext) && !f.moduleId && ((_b = (_a = f.path) === null || _a === void 0 ? void 0 : _a.startsWith) === null || _b === void 0 ? void 0 : _b.call(_a, 'materials/'))) {
                    return true;
                }
                return false;
            })
                .map((f) => f.path);
            return Promise.all(filesToDelete.map(async (f) => {
                await fs_extra_1.default.remove(path_1.default.join(this.mpCIProject.projectPath, f));
                return f;
            }));
        }
        else {
            throw new Error('小程序代码静态分析失败');
        }
    }
    /**
     * 小程序-预览
     */
    async _ciPreview(options = {}) {
        var _a;
        // 需要暂时关掉 stdout, 避免 miniprogram-ci 的内容打印到控制台
        // this.api.runtime.isLocal() && this.api.console.pause();
        !process.env.CLOUDBASE_CIID && pauseConsoleOutput();
        const previewOptions = {
            version: '0.0.1',
            desc: 'CloudBase Framework 一键预览',
            pagePath: 'pages/index/index',
            searchQuery: '',
            scene: 1011,
            qrcodeFormat: 'image',
            qrcodeOutputPath: './qrcode.jpg',
            ...(((_a = this.resolvedInputs.miniprogramOptions) === null || _a === void 0 ? void 0 : _a.previewOptions) || {}),
            ...options,
        };
        const qrcodeOutputDest = path_1.default.resolve(this.resolvedInputs.outputPath, previewOptions.qrcodeOutputPath);
        const result = await (0, miniprogram_ci_1.preview)({
            project: this.mpCIProject,
            ...previewOptions,
            qrcodeOutputDest,
        }).catch((err) => {
            return err;
        });
        // this.api.runtime.isLocal() && this.api.console.resume();
        !process.env.CLOUDBASE_CIID && resumeConsoleOutput();
        if (result === null || result === void 0 ? void 0 : result.subPackageInfo) {
            return {
                link: qrcodeOutputDest,
            };
        }
        else {
            throw new Error(`小程序（预览版）部署失败 ${result}`);
        }
    }
    /**
     * 小程序-上传
     */
    async _ciUpload(options = {}) {
        // 需要暂时关掉 stdout, 避免 miniprogram-ci 的内容打印到控制台
        // this.api.runtime.isLocal() && this.api.console.pause();
        !process.env.CLOUDBASE_CIID && pauseConsoleOutput();
        const uploadOptions = {
            version: '1.0.0',
            desc: 'CloudBase Framework 一键上传',
            ...(this.resolvedInputs.miniprogramOptions.uploadOptions || {}),
            ...(options || {}),
        };
        const result = await (0, miniprogram_ci_1.upload)({
            project: this.mpCIProject,
            ...uploadOptions,
        }).catch((err) => {
            return err;
        });
        // this.api.runtime.isLocal() && this.api.console.resume();
        !process.env.CLOUDBASE_CIID && resumeConsoleOutput();
        if (result === null || result === void 0 ? void 0 : result.subPackageInfo) {
            return {
                version: uploadOptions.version,
            };
        }
        else {
            throw new Error(`小程序（体验版）部署失败 ${result}`);
        }
    }
}
exports.default = LowcodeDeployer;
exports.deployer = LowcodeDeployer;
const originalStdoutWrite = process.stdout.write.bind(process.stdout);
const originalStderrWrite = process.stderr.write.bind(process.stderr);
let previousStdoutWrite = process.stdout.write.bind(process.stdout);
let previousStderrWrite = process.stderr.write.bind(process.stderr);
// 暂停控制台输出
function pauseConsoleOutput() {
    previousStdoutWrite = process.stdout.write.bind(process.stdout);
    process.stdout.write = () => {
        return true;
    };
    previousStderrWrite = process.stderr.write.bind(process.stderr);
    process.stderr.write = () => {
        return true;
    };
}
// 恢复控制台输出
function resumeConsoleOutput(original = false) {
    if (original) {
        process.stdout.write = originalStdoutWrite;
        process.stderr.write = originalStderrWrite;
    }
    else {
        process.stdout.write = previousStdoutWrite;
        process.stderr.write = previousStderrWrite;
    }
}
function zipDir(src, dist) {
    return new Promise((resolve, reject) => {
        // create a file to stream archive data to.
        let output = fs_extra_1.default.createWriteStream(dist);
        let archive = (0, archiver_1.default)('zip', {
            zlib: { level: 9 }, // Sets the compression level.
        });
        output.on('close', resolve);
        archive.on('error', reject);
        archive.glob('**/*', {
            cwd: src,
            ignore: [dist],
            dot: true,
        }, {});
        archive.pipe(output);
        archive.finalize();
    });
}
