import COS from 'cos-nodejs-sdk-v5';
import S3 from 'aws-sdk/clients/s3';
import { ICredentialOptions } from '../../types';
import { Readable } from 'stream';
import fs from 'fs-extra';
import path from 'path';
import { lookup as lookupMIME } from 'mime-types';

export class Storage {
  private _instance: COS | S3;
  private _cos: boolean;
  private _region: string;
  constructor({
    endpoint,
    region,
    credential,
    cos = true,
  }: {
    credential: ICredentialOptions;
    endpoint?: string;
    region?: string;
    cos?: boolean;
  }) {
    this._cos = cos;
    this._region = region || '';

    if (this._cos) {
      this._instance = credential?.token
        ? new COS({
            getAuthorization(options, callback) {
              callback({
                TmpSecretId: credential?.secretId || '',
                TmpSecretKey: credential?.secretKey || '',
                XCosSecurityToken: credential?.token || '',
                ExpiredTime: Math.floor(Date.now() / 1000) + 600,
                StartTime: Math.floor(Date.now() / 1000),
              } as any);
            },
          })
        : new COS({
            SecretId: credential?.secretId,
            SecretKey: credential?.secretKey,
          });
    } else {
      this._instance = new S3({
        endpoint: endpoint || 'http://oss-csp1.rgw.infra.tce.io',
        region: region || 'us-east-1',
        s3ForcePathStyle: true,
        sslEnabled: false,
        credentials: {
          accessKeyId: credential.secretId,
          secretAccessKey: credential.secretKey,
          sessionToken: credential.token || '',
        },
      });
    }
  }

  async getObjectSignedUrl({ Bucket, Key, Expires }): Promise<{ Url: string }> {
    const baseParam = { Bucket, Key, Expires };
    if (!this._cos) {
      const url = await (this._instance as S3).getSignedUrl('getObject', { ...baseParam });
      return { Url: url };
    } else {
      return new Promise((resolve, reject) => {
        (this._instance as COS).getObjectUrl(
          {
            ...baseParam,
            Sign: true,
            Region: this._region || 'ap-shanghai',
          },
          (err, data) => {
            if (err) {
              reject(err);
            } else {
              resolve(data);
            }
          },
        );
      });
    }
  }

  async putObject({
    Bucket,
    Key,
    Body,
    ACL,
    Metadata,
  }: {
    Bucket: string;
    Key: string;
    Body: Buffer | String | Readable;
    ACL?: S3.PutObjectRequest['ACL'];
    Metadata?: S3.PutObjectRequest['Metadata'];
  }): Promise<null> {
    return new Promise((resolve, reject) => {
      const baseParam = {
        Bucket,
        Body,
        Key,
      };
      if (this._cos) {
        (this._instance as COS).putObject(
          {
            Region: this._region || 'ap-shanghai',
            ...baseParam,
          },
          (err) => {
            if (err) {
              reject(err);
            } else {
              resolve(null);
            }
          },
        );
      } else {
        const contentType = lookupMIME(baseParam.Key);
        (this._instance as S3).putObject(
          {
            ACL,
            ContentType: contentType || 'application/octet-stream',
            Metadata,
            ...baseParam,
          },
          (err) => {
            if (err) {
              reject(err);
            } else {
              resolve(null);
            }
          },
        );
      }
    });
  }

  async putBucketAcl({ ACL, Bucket }) {
    const baseParam = {
      Bucket,
    };
    return new Promise((resolve, reject) => {
      if (this._cos) {
        throw new Error('');
      } else {
        (this._instance as S3).putBucketAcl(
          {
            ACL,
            ...baseParam,
          },
          (err) => {
            if (err) {
              reject(err);
            } else {
              resolve(null);
            }
          },
        );
      }
    });
  }

  async putObjectAcl({ ACL, Bucket, Key }) {
    const baseParam = {
      Bucket,
      Key,
    };
    return new Promise((resolve, reject) => {
      if (this._cos) {
        throw new Error('');
      } else {
        (this._instance as S3).putObjectAcl(
          {
            ACL,
            ...baseParam,
          },
          (err) => {
            if (err) {
              reject(err);
            } else {
              resolve(null);
            }
          },
        );
      }
    });
  }
}

export function getUploadDirFileList(src: string, cloudPath: string) {
  const stat = fs.statSync(src);
  let uploadFilePathList: { src: string; cloudPath: string }[] = [];
  if (stat.isFile()) {
    uploadFilePathList.push({
      src,
      cloudPath,
    });
  } else if (stat.isDirectory()) {
    const files = fs.readdirSync(src);
    for (const file of files) {
      uploadFilePathList = uploadFilePathList.concat(
        getUploadDirFileList(path.join(src, file), path.posix.join(cloudPath, file)),
      );
    }
  }
  return uploadFilePathList;
}
